@@ -1,22 +1,24 @@
 <?php
-/* Icinga Web 2 | (c) 2013-2015 Icinga Development Team | GPLv2+ */
+/* Icinga Web 2 | (c) 2013 Icinga Development Team | GPLv2+ */
 
 namespace Icinga\Protocol\Ldap;
 
-use Exception;
 use ArrayIterator;
+use Exception;
+use LogicException;
+use stdClass;
 use Icinga\Application\Config;
 use Icinga\Application\Logger;
 use Icinga\Data\ConfigObject;
+use Icinga\Data\Filter\Filter;
+use Icinga\Data\Filter\FilterChain;
+use Icinga\Data\Filter\FilterExpression;
 use Icinga\Data\Inspectable;
 use Icinga\Data\Inspection;
 use Icinga\Data\Selectable;
 use Icinga\Data\Sortable;
-use Icinga\Data\Filter\Filter;
-use Icinga\Data\Filter\FilterChain;
-use Icinga\Data\Filter\FilterExpression;
 use Icinga\Exception\ProgrammingError;
-use Icinga\Protocol\Ldap\LdapException;
+use Icinga\Web\Url;
 
 /**
  * Encapsulate LDAP connections and query creation
@@ -151,6 +153,13 @@ class LdapConnection implements Selectable, Inspectable
      */
     protected $discoverySuccess;
 
+    /**
+     * The cause of the discovery's failure
+     *
+     * @var Exception|null
+     */
+    private $discoveryError;
+
     /**
      * Whether the current connection is encrypted
      *
@@ -248,11 +257,13 @@ class LdapConnection implements Selectable, Inspectable
             try {
                 $this->capabilities = LdapCapabilities::discoverCapabilities($this);
                 $this->discoverySuccess = true;
+                $this->discoveryError = null;
             } catch (LdapException $e) {
                 Logger::debug($e);
                 Logger::warning('LADP discovery failed, assuming default LDAP capabilities.');
                 $this->capabilities = new LdapCapabilities(); // create empty default capabilities
                 $this->discoverySuccess = false;
+                $this->discoveryError = $e;
             }
         }
 
@@ -273,6 +284,16 @@ class LdapConnection implements Selectable, Inspectable
         return $this->discoverySuccess;
     }
 
+    /**
+     * Get discovery error if any
+     *
+     * @return Exception|null
+     */
+    public function getDiscoveryError()
+    {
+        return $this->discoveryError;
+    }
+
     /**
      * Return whether the current connection is encrypted
      *
@@ -315,11 +336,11 @@ class LdapConnection implements Selectable, Inspectable
         $success = @ldap_bind($ds, $this->bindDn, $this->bindPw);
         if (! $success) {
             throw new LdapException(
-                'LDAP connection to %s:%s (%s / %s) failed: %s',
-                $this->hostname,
-                $this->port,
+                'LDAP bind (%s / %s) to %s with default port %s failed: %s',
                 $this->bindDn,
                 '***' /* $this->bindPw */,
+                $this->hostname,
+                $this->port,
                 ldap_error($ds)
             );
         }
@@ -406,10 +427,7 @@ class LdapConnection implements Selectable, Inspectable
     {
         $this->bind();
 
-        if ($query->getUsePagedResults()
-            && version_compare(PHP_VERSION, '5.4.0') >= 0
-            && $this->getCapabilities()->hasPagedResult()
-        ) {
+        if ($query->getUsePagedResults() && $this->getCapabilities()->hasPagedResult()) {
             return $this->runPagedQuery($query, $fields);
         } else {
             return $this->runQuery($query, $fields);
@@ -548,6 +566,23 @@ class LdapConnection implements Selectable, Inspectable
         return $pairs;
     }
 
+    /**
+     * Fetch an LDAP entry by its DN
+     *
+     * @param  string        $dn
+     * @param  array|null    $fields
+     *
+     * @return StdClass|bool
+     */
+    public function fetchByDn($dn, array $fields = null)
+    {
+        return $this->select()
+            ->from('*', $fields)
+            ->setBase($dn)
+            ->setScope('base')
+            ->fetchRow();
+    }
+
     /**
      * Test the given LDAP credentials by establishing a connection and attempting a LDAP bind
      *
@@ -706,7 +741,7 @@ class LdapConnection implements Selectable, Inspectable
                         'value' => $this->encodeSortRules($query->getOrder())
                     )
                 ));
-            } else {
+            } elseif (! empty($fields)) {
                 foreach ($query->getOrder() as $rule) {
                     if (! in_array($rule[0], $fields, true)) {
                         $fields[] = $rule[0];
@@ -729,7 +764,7 @@ class LdapConnection implements Selectable, Inspectable
             $query,
             array_values($fields),
             0,
-            $serverSorting && $limit ? $offset + $limit : 0
+            ($serverSorting || ! $query->hasOrder()) && $limit ? $offset + $limit : 0
         );
         if ($results === false) {
             if (ldap_errno($ds) === self::LDAP_NO_SUCH_OBJECT) {
@@ -785,8 +820,11 @@ class LdapConnection implements Selectable, Inspectable
             && ($entry = ldap_next_entry($ds, $entry))
         );
 
-        if (! $serverSorting && $query->hasOrder()) {
-            uasort($entries, array($query, 'compare'));
+        if (! $serverSorting) {
+            if ($query->hasOrder()) {
+                uasort($entries, array($query, 'compare'));
+            }
+
             if ($limit && $count > $limit) {
                 $entries = array_splice($entries, $query->hasOffset() ? $query->getOffset() : 0, $limit);
             }
@@ -825,7 +863,7 @@ class LdapConnection implements Selectable, Inspectable
         $ds = $this->getConnection();
 
         $serverSorting = false;//$this->getCapabilities()->hasOid(LdapCapabilities::LDAP_SERVER_SORT_OID);
-        if (! $serverSorting && $query->hasOrder()) {
+        if (! $serverSorting && $query->hasOrder() && ! empty($fields)) {
             foreach ($query->getOrder() as $rule) {
                 if (! in_array($rule[0], $fields, true)) {
                     $fields[] = $rule[0];
@@ -864,7 +902,7 @@ class LdapConnection implements Selectable, Inspectable
                 $query,
                 array_values($fields),
                 0,
-                $serverSorting && $limit ? $offset + $limit : 0
+                ($serverSorting || ! $query->hasOrder()) && $limit ? $offset + $limit : 0
             );
             if ($results === false) {
                 if (ldap_errno($ds) === self::LDAP_NO_SUCH_OBJECT) {
@@ -926,8 +964,7 @@ class LdapConnection implements Selectable, Inspectable
                         );
                     }
                 }
-            } while (
-                (! $serverSorting || $limit === 0 || $limit !== count($entries))
+            } while ((! $serverSorting || $limit === 0 || $limit !== count($entries))
                 && ($entry = ldap_next_entry($ds, $entry))
             );
 
@@ -957,8 +994,11 @@ class LdapConnection implements Selectable, Inspectable
             ldap_search($ds, $query->getBase() ?: $this->getDn(), (string) $query);
         }
 
-        if (! $serverSorting && $query->hasOrder()) {
-            uasort($entries, array($query, 'compare'));
+        if (! $serverSorting) {
+            if ($query->hasOrder()) {
+                uasort($entries, array($query, 'compare'));
+            }
+
             if ($limit && $count > $limit) {
                 $entries = array_splice($entries, $query->hasOffset() ? $query->getOffset() : 0, $limit);
             }
@@ -1035,8 +1075,7 @@ class LdapConnection implements Selectable, Inspectable
             }
         }
 
-        if (
-            $unfoldAttribute !== null
+        if ($unfoldAttribute !== null
             && isset($cleanedAttributes[$unfoldAttribute])
             && is_array($cleanedAttributes[$unfoldAttribute])
         ) {
@@ -1092,7 +1131,6 @@ class LdapConnection implements Selectable, Inspectable
             if ($attributeOctets >= 127) {
                 // Use the indefinite form of the length octets (the long form would be another option)
                 $attributeType = '0440' . $attributeType . '0000';
-
             } else {
                 $attributeType = '04' . str_pad(dechex($attributeOctets), 2, '0', STR_PAD_LEFT) . $attributeType;
             }
@@ -1115,11 +1153,7 @@ class LdapConnection implements Selectable, Inspectable
             $sequenceOf = '30' . str_pad(dechex($sequenceOfOctets), 2, '0', STR_PAD_LEFT) . $sequenceOf;
         }
 
-        if (version_compare(PHP_VERSION, '5.4.0') >= 0) {
-            return hex2bin($sequenceOf);
-        } else {
-            return pack('H*', $sequenceOf);
-        }
+        return hex2bin($sequenceOf);
     }
 
     /**
@@ -1140,7 +1174,18 @@ class LdapConnection implements Selectable, Inspectable
         $hostname = $this->hostname;
         if ($this->encryption === static::LDAPS) {
             $info->write('Connect using LDAPS');
-            $hostname = 'ldaps://' . $hostname;
+            $ldapUrls = explode(' ', $hostname);
+            if (count($ldapUrls) > 1) {
+                foreach ($ldapUrls as & $uri) {
+                    if (strpos($uri, '://') === false) {
+                        $uri = 'ldaps://' . $uri;
+                    }
+                }
+
+                $hostname = implode(' ', $ldapUrls);
+            } else {
+                $hostname = 'ldaps://' . $hostname;
+            }
         }
 
         $ds = ldap_connect($hostname, $this->port);
@@ -1159,7 +1204,6 @@ class LdapConnection implements Selectable, Inspectable
             if (! ldap_start_tls($ds)) {
                 throw new LdapException('LDAP STARTTLS failed: %s', ldap_error($ds));
             }
-
         } elseif ($this->encryption !== static::LDAPS) {
             $this->encrypted = false;
             $info->write('Connect without encryption');
@@ -1179,6 +1223,8 @@ class LdapConnection implements Selectable, Inspectable
      * @param   int         $deref
      *
      * @return  resource|bool               A search result identifier or false on error
+     *
+     * @throws  LogicException              If the LDAP query search scope is unsupported
      */
     public function ldapSearch(
         LdapQuery $query,
@@ -1190,16 +1236,32 @@ class LdapConnection implements Selectable, Inspectable
     ) {
         $queryString = (string) $query;
         $baseDn = $query->getBase() ?: $this->getDn();
+        $scope = $query->getScope();
 
         if (Logger::getInstance()->getLevel() === Logger::DEBUG) {
-            // We're checking the level by ourself to avoid rendering the ldapsearch commandline for nothing
+            // We're checking the level by ourselves to avoid rendering the ldapsearch commandline for nothing
             $starttlsParam = $this->encryption === static::STARTTLS ? ' -ZZ' : '';
-            $ldapUrl = ($this->encryption === static::LDAPS ? 'ldaps://' : 'ldap://')
-                . $this->hostname
-                . ($this->port ? ':' . $this->port : '');
 
+            $ldapUrls = array();
+            $defaultScheme = $this->encryption === static::LDAPS ? 'ldaps://' : 'ldap://';
+            foreach (explode(' ', $this->hostname) as $uri) {
+                $url = Url::fromPath($uri);
+                if (! $url->getScheme()) {
+                    $uri = $defaultScheme . $uri . ($this->port ? ':' . $this->port : '');
+                } else {
+                    if ($url->getPort() === null) {
+                        $url->setPort($this->port);
+                    }
+
+                    $uri = $url->getAbsoluteUrl();
+                }
+
+                $ldapUrls[] = $uri;
+            }
+
+            $bindParams = '';
             if ($this->bound) {
-                $bindParams = ' -D "' . $this->bindDn . '"' . ($this->bindPw ? ' -w "' . $this->bindPw . '"' : '');
+                $bindParams = ' -D "' . $this->bindDn . '"' . ($this->bindPw ? ' -W' : '');
             }
 
             if ($deref === LDAP_DEREF_NEVER) {
@@ -1213,11 +1275,12 @@ class LdapConnection implements Selectable, Inspectable
             }
 
             Logger::debug("Issueing LDAP search. Use '%s' to reproduce.", sprintf(
-                'ldapsearch -P 3%s -H "%s"%s -b "%s" -s "sub" -z %u -l %u -a "%s"%s%s%s',
+                'ldapsearch -P 3%s -H "%s"%s -b "%s" -s "%s" -z %u -l %u -a "%s"%s%s%s',
                 $starttlsParam,
-                $ldapUrl,
+                implode(' ', $ldapUrls),
                 $bindParams,
                 $baseDn,
+                $scope,
                 $sizelimit,
                 $timelimit,
                 $derefName,
@@ -1227,7 +1290,21 @@ class LdapConnection implements Selectable, Inspectable
             ));
         }
 
-        return @ldap_search(
+        switch ($scope) {
+            case LdapQuery::SCOPE_SUB:
+                $function = 'ldap_search';
+                break;
+            case LdapQuery::SCOPE_ONE:
+                $function = 'ldap_list';
+                break;
+            case LdapQuery::SCOPE_BASE:
+                $function = 'ldap_read';
+                break;
+            default:
+                throw new LogicException('LDAP scope %s not supported by ldapSearch', $scope);
+        }
+
+        return @$function(
             $this->getConnection(),
             $baseDn,
             $queryString,
@@ -1420,11 +1497,11 @@ class LdapConnection implements Selectable, Inspectable
         // Try a bind-command with the given user credentials, this must not fail
         $success = @ldap_bind($ds, $this->bindDn, $this->bindPw);
         $msg = sprintf(
-            'LDAP bind to %s:%s (%s / %s)',
-            $this->hostname,
-            $this->port,
+            'LDAP bind (%s / %s) to %s with default port %s',
             $this->bindDn,
-            '***' /* $this->bindPw */
+            '***' /* $this->bindPw */,
+            $this->hostname,
+            $this->port
         );
         if (! $success) {
             // ldap_error does not return any proper error messages in case of certificate errors. Connecting
