@@ -90,8 +90,7 @@ class DocParser
                 return null;
             }
             $headerStyle = static::HEADER_ATX;
-        } elseif (
-            $nextLine
+        } elseif ($nextLine
             && ($nextLine[0] === '=' || $nextLine[0] === '-')
             && preg_match('/^[=-]+\s*$/', $nextLine, $match) === 1
         ) {
@@ -110,7 +109,7 @@ class DocParser
         if ($header === null) {
             return null;
         }
-        if ($header[0] === '<'
+        if (strpos($header, '<') !== false
             && preg_match('#(?:<(?P<tag>a|span) (?:id|name)="(?P<id>.+)"></(?P=tag)>)\s*#u', $header, $match)
         ) {
             $header = str_replace($match[0], '', $header);
@@ -162,10 +161,19 @@ class DocParser
             $lastLine = null;
             $stack = new SplStack();
             $cachingIterator = new CachingIterator($file, CachingIterator::TOSTRING_USE_CURRENT);
+            $insideFencedCodeBlock = false;
+
             for ($cachingIterator->rewind(); $line = $cachingIterator->valid(); $cachingIterator->next()) {
                 $fileIterator = $cachingIterator->getInnerIterator();
                 $line = $cachingIterator->current();
-                $header = $this->extractHeader($line, $fileIterator->valid() ? $fileIterator->current() : null);
+                $header = null;
+
+                if (substr($line, 0, 3) === '```') {
+                    $insideFencedCodeBlock = ! $insideFencedCodeBlock;
+                } elseif (! $insideFencedCodeBlock) {
+                    $header = $this->extractHeader($line, $fileIterator->valid() ? $fileIterator->current() : null);
+                }
+
                 if ($header !== null) {
                     list($title, $id, $level, $headerStyle) = $header;
                     while (! $stack->isEmpty() && $stack->top()->getLevel() >= $level) {
