@@ -2,6 +2,7 @@
 /* Icinga Web 2 | (c) 2013 Icinga Development Team | GPLv2+ */
 
 use Icinga\Web\Dom\DomNodeIterator;
+use Icinga\Module\Monitoring\Web\Helper\PluginOutputPurifier;
 
 /**
  * Plugin output renderer
@@ -28,6 +29,8 @@ class Zend_View_Helper_PluginOutput extends Zend_View_Helper_Abstract
         '~(\[|\()WARNING(\]|\))~',
         '~(\[|\()CRITICAL(\]|\))~',
         '~(\[|\()UNKNOWN(\]|\))~',
+        '~(\[|\()UP(\]|\))~',
+        '~(\[|\()DOWN(\]|\))~',
         '~\@{6,}~'
     );
 
@@ -44,29 +47,67 @@ class Zend_View_Helper_PluginOutput extends Zend_View_Helper_Abstract
         '<span class="state-warning">$1WARNING$2</span>',
         '<span class="state-critical">$1CRITICAL$2</span>',
         '<span class="state-unknown">$1UNKNOWN$2</span>',
+        '<span class="state-up">$1UP$2</span>',
+        '<span class="state-down">$1DOWN$2</span>',
         '@@@@@@',
     );
 
+    /**
+     * Patterns to be replaced in html plugin output
+     *
+     * @var array
+     */
+    protected static $htmlPatterns = array(
+        '~\\\n~',
+        '~\\\t~',
+        '~\\\n\\\n~',
+        '~<table~'
+    );
+
+    /**
+     * Replacements for $htmlPatterns
+     *
+     * @var array
+     */
+    protected static $htmlReplacements = array(
+        "\n",
+        "\t",
+        "\n",
+        '<table style="font-size: 0.75em"'
+    );
+
+    /** @var \Icinga\Module\Monitoring\Web\Helper\PluginOutputHookRenderer */
+    protected $hookRenderer;
+
+    public function __construct()
+    {
+        $this->hookRenderer = (new \Icinga\Module\Monitoring\Web\Helper\PluginOutputHookRenderer())->registerHooks();
+    }
+
     /**
      * Render plugin output
      *
      * @param   string  $output
      * @param   bool    $raw
+     * @param   string  $command    Check command
      *
      * @return  string
      */
-    public function pluginOutput($output, $raw = false)
+    public function pluginOutput($output, $raw = false, $command = null)
     {
         if (empty($output)) {
             return '';
         }
+        if ($command !== null) {
+            $output = $this->hookRenderer->render($command, $output, ! $raw);
+        }
         $output = preg_replace('~<br[^>]*>~', "\n", $output);
         if (preg_match('~<[^>]*["/\'][^>]*>~', $output)) {
             // HTML
             $output = preg_replace(
-                '~<table~',
-                '<table style="font-size: 0.75em"',
-                $this->getPurifier()->purify($output)
+                self::$htmlPatterns,
+                self::$htmlReplacements,
+                PluginOutputPurifier::process($output)
             );
             $isHtml = true;
         } else {
@@ -101,7 +142,7 @@ class Zend_View_Helper_PluginOutput extends Zend_View_Helper_Abstract
      */
     protected function processHtml($html)
     {
-        $pattern = '/[([](OK|WARNING|CRITICAL|UNKNOWN)[)\]]/';
+        $pattern = '/[([](OK|WARNING|CRITICAL|UNKNOWN|UP|DOWN)[)\]]/';
         $doc = new DOMDocument();
         $doc->loadXML('<div>' . $html . '</div>', LIBXML_NOERROR | LIBXML_NOWARNING);
         $dom = new RecursiveIteratorIterator(new DomNodeIterator($doc), RecursiveIteratorIterator::SELF_FIRST);
@@ -114,16 +155,28 @@ class Zend_View_Helper_PluginOutput extends Zend_View_Helper_Abstract
                     $offsetLeft = $match[0][1];
                     $matchLength = strlen($match[0][0]);
                     $leftLength = $offsetLeft - $start;
+                    // if there is text before the match
                     if ($leftLength) {
+                        // create node for leading text
                         $text = new DOMText(substr($node->nodeValue, $start, $leftLength));
                         $node->parentNode->insertBefore($text, $node);
                     }
+                    // create the new element for the match
                     $span = $doc->createElement('span', $match[0][0]);
                     $span->setAttribute('class', 'state-' . strtolower($match[1][0]));
                     $node->parentNode->insertBefore($span, $node);
+
+                    // start for next match
                     $start = $offsetLeft + $matchLength;
                 }
                 if ($start) {
+                    // is there text left?
+                    if (strlen($node->nodeValue) > $start) {
+                        // create node for trailing text
+                        $text = new DOMText(substr($node->nodeValue, $start));
+                        $node->parentNode->insertBefore($text, $node);
+                    }
+                    // delete the old node later
                     $nodesToRemove[] = $node;
                 }
             } elseif ($node->nodeType === XML_ELEMENT_NODE) {
@@ -148,34 +201,4 @@ class Zend_View_Helper_PluginOutput extends Zend_View_Helper_Abstract
 
         return substr($doc->saveHTML(), 5, -7);
     }
-
-    /**
-     * Initialize and return self::$purifier
-     *
-     * @return HTMLPurifier
-     */
-    protected function getPurifier()
-    {
-        if (self::$purifier === null) {
-            require_once 'HTMLPurifier/Bootstrap.php';
-            require_once 'HTMLPurifier.php';
-            require_once 'HTMLPurifier.autoload.php';
-
-            $config = HTMLPurifier_Config::createDefault();
-            $config->set('Core.EscapeNonASCIICharacters', true);
-            $config->set('HTML.Allowed', 'p,br,b,a[href|target],i,table,tr,th[colspan],td[colspan],div,*[class]');
-            $config->set('Attr.AllowedFrameTargets', array('_blank'));
-            // This avoids permission problems:
-            // $config->set('Core.DefinitionCache', null);
-            $config->set('Cache.DefinitionImpl', null);
-            // TODO: Use a cache directory:
-            // $config->set('Cache.SerializerPath', '/var/spool/whatever');
-
-            // $config->set('URI.Base', 'http://www.example.com');
-            // $config->set('URI.MakeAbsolute', true);
-            // $config->set('AutoFormat.AutoParagraph', true);
-            self::$purifier = new HTMLPurifier($config);
-        }
-        return self::$purifier;
-    }
 }
